"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const Validator_1 = __importDefault(require("./Validator"));
const util_1 = require("../util");
const qs_1 = __importDefault(require("qs"));
const validator = Validator_1.default;
const UNPROCESSABLE_ENTITY = 422;
class BaseProxy {
    constructor(endpoint, parameters) {
        this.endpoint = endpoint;
        this.parameters = parameters;
        this.errors = Validator_1.default;
    }
    get $http() {
        return BaseProxy.$http;
    }
    get $errorProperty() {
        return BaseProxy.$errorProperty;
    }
    /**
     * Get all or by pagination
     */
    all() {
        return this.submit('get');
    }
    /**
     * Alternative of all method
     */
    getMany() {
        return this.all();
    }
    /**
     * Find a record by id
     * @param {string|number} id
     */
    find(id) {
        return this.submit('get', id);
    }
    /**
     * Alternative of find method
     * @param {string | number} id
     */
    getOne(id) {
        return this.find(id);
    }
    /**
     * Create record
     * @param {Object|string} payload
     * @param {AxiosRequestConfig} config
     */
    post(payload, config) {
        return this.submit('post', '', payload, config);
    }
    /**
     * Alternative of post method
     * @param payload
     * @param {AxiosRequestConfig} config
     */
    store(payload, config) {
        return this.post(payload, config);
    }
    /**
     * Alternative of store method
     * @param payload
     * @param {AxiosRequestConfig} config
     */
    create(payload, config) {
        return this.store(payload, config);
    }
    /**
     * Create many items
     * @param {Object} payload
     */
    createMany(payload) {
        return this.submit('post', 'bulk', payload);
    }
    /**
     * Update record by id using PUT method
     * @param {string|number} id
     * @param {Object|string} payload
     */
    put(id, payload) {
        return this.submit('put', `/${id}`, payload);
    }
    /**
     * Alternative of put method
     * @param {string|number} id
     * @param {Object|string} payload
     */
    replace(id, payload) {
        return this.put(id, payload);
    }
    /**
     * This method helpful for laravel developer
     * @param {string|number} id
     * @param {Object} payload
     * @param config
     */
    putWithFile(id, payload, config) {
        payload._method = 'put';
        return this.submit('post', `/${id}`, payload, config);
    }
    /**
     * Update record by id
     * @param id
     * @param payload
     */
    patch(id, payload) {
        return this.submit('patch', `/${id}`, payload);
    }
    /**
     * Alternative of path method
     * @param {string|number} id
     * @param {Object|string} payload
     */
    update(id, payload) {
        return this.patch(id, payload);
    }
    /**
     * Delete record by id
     * @param {string|number} id
     */
    delete(id) {
        return this.submit('delete', `/${id}`);
    }
    /**
     * Alternative of delete method
     * @param {string|number} id
     */
    remove(id) {
        return this.delete(id);
    }
    /**
     * Main endpoint method to handle requests
     * @param requestType
     * @param {string} parameter
     * @param {Object|string} form
     * @param {AxiosRequestConfig} config
     */
    submit(requestType, parameter, form, config) {
        const method = BaseProxy.__validateRequestType(requestType);
        this.beforeSubmit();
        return new Promise((resolve, reject) => {
            const data = (0, util_1.hasFiles)(form) ? (0, util_1.objectToFormData)(form) : form;
            const url = parameter
                ? `/${this.endpoint}/${parameter}`
                : `/${this.endpoint}`;
            const endpoint = this.__getParameterString((0, util_1.removeDoubleSlash)(url));
            // eslint-disable-next-line @typescript-eslint/ban-ts-comment
            // @ts-ignore
            this.$http[method](endpoint, data, config)
                .then((response) => {
                this.onSuccess();
                const { data } = response;
                resolve(data);
            })
                .catch((error) => {
                this.errors.processing = false;
                validator.processing = false;
                const { response } = error;
                if (response) {
                    const { data, status } = response;
                    if (status === UNPROCESSABLE_ENTITY) {
                        const errors = {};
                        Object.assign(errors, data[this.$errorProperty]);
                        this.onFail(errors);
                    }
                    reject(error);
                }
                else {
                    reject(error);
                }
            });
        });
    }
    __getParameterString(url) {
        const query = qs_1.default.stringify(this.parameters, {
            encode: false,
            skipNulls: true,
            addQueryPrefix: true,
        });
        return `${url}${query}`;
    }
    static __validateRequestType(requestType) {
        const requestTypes = [
            'get',
            'delete',
            'head',
            'post',
            'put',
            'patch',
        ];
        if (!requestTypes.includes(requestType)) {
            throw new Error(`\`${requestType}\` is not a valid request type, ` +
                `must be one of: \`${requestTypes.join('`, `')}\`.`);
        }
        return requestType.toLowerCase();
    }
    /**
     * Set parameters by keys
     * @param {Object} parameters
     */
    setParameters(parameters) {
        Object.keys(parameters).forEach((key) => {
            this.parameters[key] = parameters[key];
        });
        return this;
    }
    /**
     * Set parameters by key
     * @param {string} parameter
     * @param {Object|string|Array} value
     */
    setParameter(parameter, value) {
        if (!value) {
            const options = {
                comma: true,
                allowDots: true,
                ignoreQueryPrefix: true,
            };
            const params = qs_1.default.parse(parameter, options);
            return this.setParameters(params);
        }
        this.parameters[parameter] = value;
        return this;
    }
    /**
     * Remove parameters by keys
     * @param {Array<Object>>} parameters
     */
    removeParameters(parameters = []) {
        if (!parameters.length) {
            this.parameters = [];
        }
        else {
            parameters.forEach((parameter) => {
                delete this.parameters[parameter];
            });
        }
        return this;
    }
    /**
     * Remove parameters
     * @param {string} parameter
     */
    removeParameter(parameter) {
        delete this.parameters[parameter];
        return this;
    }
    /**
     * Fill errors on fails passed
     * @param {Object} errors
     */
    onFail(errors) {
        this.errors.fill(errors);
        validator.fill(errors);
    }
    /**
     * Clean up errors and set processing
     */
    beforeSubmit() {
        if (!this.$http) {
            throw new Error('Vue Api Queries, No http library provided.');
        }
        this.errors.flush();
        this.errors.processing = true;
        this.errors.successful = false;
        validator.flush();
        validator.processing = true;
        validator.successful = false;
    }
    /**
     * Clean up errors and set success on after request
     */
    onSuccess() {
        this.errors.processing = false;
        this.errors.successful = true;
        validator.processing = false;
        validator.successful = true;
    }
}
BaseProxy.$errorProperty = 'errors';
exports.default = BaseProxy;
